@@ -4,7 +4,7 @@
 
 plot.stslist.rep <- function(x, cpal = NULL, missing.color = NULL, pbarw = TRUE,
   dmax = NULL, stats = TRUE, ylab = NULL, xaxis = TRUE, xtlab = NULL,
-  xtstep = NULL, cex.with.axis = 1, cex.plot, ...) {
+  xtstep = NULL, tick.last = NULL, cex.with.axis = 1, cex.plot, ...) {
 
   TraMineR.check.depr.args(alist(cex.with.axis = cex.plot))
 
@@ -18,6 +18,9 @@ plot.stslist.rep <- function(x, cpal = NULL, missing.color = NULL, pbarw = TRUE,
 		## For sequence objects created with previous versions
 		else {xtstep <- 1}
 	}
+	if(is.null(tick.last)){
+		tick.last <- ifelse(!is.null(attr(x, "tick.last")), attr(x, "tick.last"), FALSE)
+	}
 
 	weighted <- attr(x, "weighted")
 	if (weighted) {wlab <- "weighted "}
@@ -95,6 +98,7 @@ plot.stslist.rep <- function(x, cpal = NULL, missing.color = NULL, pbarw = TRUE,
 	## Time axis for the sequence
 	if (xaxis) {
 		tpos <- seq(1,seql, xtstep)
+    if (tick.last & tpos[length(tpos)] < seql) tpos <- c(tpos,seql)
 
 		axis(1, at=tpos-0.5, labels=xtlab[tpos],
 			pos=-0.04,
@@ -185,4 +189,3 @@ plot.stslist.rep <- function(x, cpal = NULL, missing.color = NULL, pbarw = TRUE,
 			cex=cex.with.axis)
 	}
 }
-
